from ._util import launcherctl


class App:
    def __init__(self, name: str):
        self.name = name

    @property
    def is_running(self) -> bool:
        return self.name in api.running.keys()

    @property
    def is_paused(self) -> bool:
        return self.name in api.paused.keys()

    def start(self) -> None:
        launcherctl("start-app", self.name)

    def stop(self) -> None:
        launcherctl("stop-app", self.name)

    def pause(self) -> None:
        launcherctl("pause-app", self.name)

    def resume(self) -> None:
        launcherctl("resume-app", self.name)


class API:
    def keys(self) -> list[str]:
        return [x.decode("utf-8") for x in launcherctl("list-apps").splitlines()]

    def __contains__(self, key: str) -> bool:
        return key in self.keys()

    def __getitem__(self, key: str) -> App:
        if key not in self:
            raise KeyError()

        return App(key)

    @property
    def running(self) -> dict[App]:
        return {
            x: App(x)
            for x in [
                x.decode("utf-8") for x in launcherctl("list-running-apps").splitlines()
            ]
        }

    @property
    def paused(self) -> dict[App]:
        return {
            x: App(x)
            for x in [
                x.decode("utf-8") for x in launcherctl("list-paused-apps").splitlines()
            ]
        }


api = API()



import subprocess


def LauncherCtlException(Exception):
    pass


def launcherctl(*args) -> str | None:
    try:
        return subprocess.check_output(["/opt/bin/launcherctl"] + list(args))
    except subprocess.CalledProcessError as e:
        stdout = e.output.decode("utf-8")
        raise LauncherCtlException(stdout) from e



import subprocess

from typing import Callable

from ._app import App
from ._util import launcherctl
from ._util import LauncherCtlException


class Launcher:
    def __init__(self, name: str):
        self.name = name

    def logs(self, onlogline: Callable[[str], None] = None) -> list[str] | None:
        pass

    def start(self):
        launcherctl("start-launcher", self.name)

    def stop(self):
        launcherctl("stop-launcher", self.name)

    def enable(self, start: bool = False):
        if start:
            launcherctl("switch-launcher", "--start", self.name)

        else:
            launcherctl("switch-launcher", self.name)

    @property
    def is_current(self) -> bool:
        return (
            subprocess.call(["/opt/bin/launcherctl", "is-current-launcher", self.name])
            == 0
        )

    @property
    def is_enabled(self) -> bool:
        return (
            subprocess.call(["/opt/bin/launcherctl", "is-enabled-launcher", self.name])
            == 0
        )

    @property
    def is_active(self) -> bool:
        return (
            subprocess.call(["/opt/bin/launcherctl", "is-active-launcher", self.name])
            == 0
        )


class API:
    def keys(self) -> list[str]:
        return [x.decode("utf-8") for x in launcherctl("list-launchers").splitlines()]

    def __contains__(self, key: str) -> bool:
        return key in self.keys()

    def __getitem__(self, key: str) -> App:
        if key not in self:
            raise KeyError()

        return Launcher(key)

    @property
    def current(self):
        return Launcher(launcherctl("status").splitlines()[0][14:-4].decode("utf-8"))

    def switch(launcher: Launcher | str, start: bool = False):
        if not isinstance(Launcher, launcher):
            launcher = Launcher(launcher)

        launcher.enable(start)


api = API()
